@@ -30,8 +30,6 @@ from chia.types.unfinished_block import UnfinishedBlock
 from chia.util import cached_bls
 from chia.util.condition_tools import (
     pkm_pairs_for_conditions_dict,
-    coin_announcements_names_for_npc,
-    puzzle_announcements_names_for_npc,
 )
 from chia.util.errors import Err
 from chia.util.generator_tools import (
@@ -159,8 +157,6 @@ async def validate_block_body(
     removals: List[bytes32] = []
     coinbase_additions: List[Coin] = list(expected_reward_coins)
     additions: List[Coin] = []
-    coin_announcement_names: Set[bytes32] = set()
-    puzzle_announcement_names: Set[bytes32] = set()
     npc_list: List[NPC] = []
     removals_puzzle_dic: Dict[bytes32, bytes32] = {}
     cost: uint64 = uint64(0)
@@ -223,8 +219,6 @@ async def validate_block_body(
             removals_puzzle_dic[npc.coin_name] = npc.puzzle_hash
 
         additions = additions_for_npc(npc_list)
-        coin_announcement_names = coin_announcements_names_for_npc(npc_list)
-        puzzle_announcement_names = puzzle_announcements_names_for_npc(npc_list)
     else:
         assert npc_result is None
 
@@ -325,7 +319,6 @@ async def validate_block_body(
                     min(constants.MAX_BLOCK_COST_CLVM, curr.transactions_info.cost),
                     cost_per_byte=constants.COST_PER_BYTE,
                     safe_mode=False,
-                    rust_checker=curr.height > constants.RUST_CONDITION_CHECKER,
                 )
                 removals_in_curr, additions_in_curr = tx_removals_and_additions(curr_npc_result.npc_list)
             else:
@@ -450,8 +443,6 @@ async def validate_block_body(
         unspent = removal_coin_records[npc.coin_name]
         error = mempool_check_conditions_dict(
             unspent,
-            coin_announcement_names,
-            puzzle_announcement_names,
             npc.condition_dict,
             prev_transaction_block_height,
             block.foliage_transaction_block.timestamp,
