@@ -1,14 +1,16 @@
-from typing import List, Optional
-
+from typing import List, Optional, Set, Dict
 import aiosqlite
-
+from chia.protocols.wallet_protocol import CoinState
 from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.coin_record import CoinRecord
-from chia.types.full_block import FullBlock
 from chia.util.db_wrapper import DBWrapper
 from chia.util.ints import uint32, uint64
 from chia.util.lru_cache import LRUCache
+from time import time
+import logging
+
+log = logging.getLogger(__name__)
 
 
 class CoinStore:
@@ -29,8 +31,8 @@ class CoinStore:
         self.cache_size = cache_size
         self.db_wrapper = db_wrapper
         self.coin_record_db = db_wrapper.db
-        await self.coin_record_db.execute("pragma journal_mode=wal")
-        await self.coin_record_db.execute("pragma synchronous=2")
+        # the coin_name is unique in this table because the CoinStore always
+        # only represent a single peak
         await self.coin_record_db.execute(
             (
                 "CREATE TABLE IF NOT EXISTS coin_record("
@@ -57,31 +59,41 @@ class CoinStore:
 
         await self.coin_record_db.execute("CREATE INDEX IF NOT EXISTS coin_puzzle_hash on coin_record(puzzle_hash)")
 
+        await self.coin_record_db.execute("CREATE INDEX IF NOT EXISTS coin_parent_index on coin_record(coin_parent)")
+
         await self.coin_record_db.commit()
         self.coin_record_cache = LRUCache(cache_size)
         return self
 
-    async def new_block(self, block: FullBlock, tx_additions: List[Coin], tx_removals: List[bytes32]):
+    async def new_block(
+        self,
+        height: uint32,
+        timestamp: uint64,
+        included_reward_coins: Set[Coin],
+        tx_additions: List[Coin],
+        tx_removals: List[bytes32],
+    ) -> List[CoinRecord]:
         """
         Only called for blocks which are blocks (and thus have rewards and transactions)
+        Returns a list of the CoinRecords that were added by this block
         """
-        if block.is_transaction_block() is False:
-            return None
-        assert block.foliage_transaction_block is not None
+
+        start = time()
+
+        additions = []
 
         for coin in tx_additions:
             record: CoinRecord = CoinRecord(
                 coin,
-                block.height,
+                height,
                 uint32(0),
                 False,
                 False,
-                block.foliage_transaction_block.timestamp,
+                timestamp,
             )
-            await self._add_coin_record(record, False)
+            additions.append(record)
 
-        included_reward_coins = block.get_included_reward_coins()
-        if block.height == 0:
+        if height == 0:
             assert len(included_reward_coins) == 0
         else:
             assert len(included_reward_coins) >= 2
@@ -89,20 +101,26 @@ class CoinStore:
         for coin in included_reward_coins:
             reward_coin_r: CoinRecord = CoinRecord(
                 coin,
-                block.height,
+                height,
                 uint32(0),
                 False,
                 True,
-                block.foliage_transaction_block.timestamp,
+                timestamp,
             )
-            await self._add_coin_record(reward_coin_r, False)
+            additions.append(reward_coin_r)
 
-        total_amount_spent: int = 0
-        for coin_name in tx_removals:
-            total_amount_spent += await self._set_spent(coin_name, block.height)
+        await self._add_coin_records(additions)
+        await self._set_spent(tx_removals, height)
 
-        # Sanity check, already checked in block_body_validation
-        assert sum([a.amount for a in tx_additions]) <= total_amount_spent
+        end = time()
+        log.log(
+            logging.WARNING if end - start > 10 else logging.DEBUG,
+            f"It took {end - start:0.2f}s to apply {len(tx_additions)} additions and "
+            + f"{len(tx_removals)} removals to the coin store. Make sure "
+            + "blockchain database is on a fast drive",
+        )
+
+        return additions
 
     # Checks DB and DiffStores for CoinRecord with coin_name and returns it
     async def get_coin_record(self, coin_name: bytes32) -> Optional[CoinRecord]:
@@ -130,6 +148,9 @@ class CoinStore:
         return coins
 
     async def get_coins_removed_at_height(self, height: uint32) -> List[CoinRecord]:
+        # Special case to avoid querying all unspent coins (spent_index=0)
+        if height == 0:
+            return []
         cursor = await self.coin_record_db.execute("SELECT * from coin_record WHERE spent_index=?", (height,))
         rows = await cursor.fetchall()
         await cursor.close()
@@ -153,7 +174,8 @@ class CoinStore:
 
         coins = set()
         cursor = await self.coin_record_db.execute(
-            f"SELECT * from coin_record WHERE puzzle_hash=? AND confirmed_index>=? AND confirmed_index<? "
+            f"SELECT * from coin_record INDEXED BY coin_puzzle_hash WHERE puzzle_hash=? "
+            f"AND confirmed_index>=? AND confirmed_index<? "
             f"{'' if include_spent_coins else 'AND spent=0'}",
             (puzzle_hash.hex(), start_height, end_height),
         )
@@ -178,7 +200,8 @@ class CoinStore:
         coins = set()
         puzzle_hashes_db = tuple([ph.hex() for ph in puzzle_hashes])
         cursor = await self.coin_record_db.execute(
-            f'SELECT * from coin_record WHERE puzzle_hash in ({"?," * (len(puzzle_hashes_db) - 1)}?) '
+            f"SELECT * from coin_record INDEXED BY coin_puzzle_hash "
+            f'WHERE puzzle_hash in ({"?," * (len(puzzle_hashes_db) - 1)}?) '
             f"AND confirmed_index>=? AND confirmed_index<? "
             f"{'' if include_spent_coins else 'AND spent=0'}",
             puzzle_hashes_db + (start_height, end_height),
@@ -192,6 +215,67 @@ class CoinStore:
             coins.add(CoinRecord(coin, row[1], row[2], row[3], row[4], row[8]))
         return list(coins)
 
+    async def get_coin_records_by_names(
+        self,
+        include_spent_coins: bool,
+        names: List[bytes32],
+        start_height: uint32 = uint32(0),
+        end_height: uint32 = uint32((2 ** 32) - 1),
+    ) -> List[CoinRecord]:
+        if len(names) == 0:
+            return []
+
+        coins = set()
+        names_db = tuple([name.hex() for name in names])
+        cursor = await self.coin_record_db.execute(
+            f'SELECT * from coin_record WHERE coin_name in ({"?," * (len(names_db) - 1)}?) '
+            f"AND confirmed_index>=? AND confirmed_index<? "
+            f"{'' if include_spent_coins else 'AND spent=0'}",
+            names_db + (start_height, end_height),
+        )
+        rows = await cursor.fetchall()
+
+        await cursor.close()
+        for row in rows:
+            coin = Coin(bytes32(bytes.fromhex(row[6])), bytes32(bytes.fromhex(row[5])), uint64.from_bytes(row[7]))
+            coins.add(CoinRecord(coin, row[1], row[2], row[3], row[4], row[8]))
+
+        return list(coins)
+
+    def row_to_coin_state(self, row):
+        coin = Coin(bytes32(bytes.fromhex(row[6])), bytes32(bytes.fromhex(row[5])), uint64.from_bytes(row[7]))
+        spent_h = None
+        if row[3]:
+            spent_h = row[2]
+        return CoinState(coin, spent_h, row[1])
+
+    async def get_coin_states_by_puzzle_hashes(
+        self,
+        include_spent_coins: bool,
+        puzzle_hashes: List[bytes32],
+        start_height: uint32 = uint32(0),
+        end_height: uint32 = uint32((2 ** 32) - 1),
+    ) -> List[CoinState]:
+        if len(puzzle_hashes) == 0:
+            return []
+
+        coins = set()
+        puzzle_hashes_db = tuple([ph.hex() for ph in puzzle_hashes])
+        cursor = await self.coin_record_db.execute(
+            f'SELECT * from coin_record WHERE puzzle_hash in ({"?," * (len(puzzle_hashes_db) - 1)}?) '
+            f"AND confirmed_index>=? AND confirmed_index<? "
+            f"{'' if include_spent_coins else 'AND spent=0'}",
+            puzzle_hashes_db + (start_height, end_height),
+        )
+
+        rows = await cursor.fetchall()
+
+        await cursor.close()
+        for row in rows:
+            coins.add(self.row_to_coin_state(row))
+
+        return list(coins)
+
     async def get_coin_records_by_parent_ids(
         self,
         include_spent_coins: bool,
@@ -219,9 +303,36 @@ class CoinStore:
             coins.add(CoinRecord(coin, row[1], row[2], row[3], row[4], row[8]))
         return list(coins)
 
-    async def rollback_to_block(self, block_index: int):
+    async def get_coin_state_by_ids(
+        self,
+        include_spent_coins: bool,
+        coin_ids: List[bytes32],
+        start_height: uint32 = uint32(0),
+        end_height: uint32 = uint32((2 ** 32) - 1),
+    ) -> List[CoinState]:
+        if len(coin_ids) == 0:
+            return []
+
+        coins = set()
+        parent_ids_db = tuple([pid.hex() for pid in coin_ids])
+        cursor = await self.coin_record_db.execute(
+            f'SELECT * from coin_record WHERE coin_name in ({"?," * (len(parent_ids_db) - 1)}?) '
+            f"AND confirmed_index>=? AND confirmed_index<? "
+            f"{'' if include_spent_coins else 'AND spent=0'}",
+            parent_ids_db + (start_height, end_height),
+        )
+
+        rows = await cursor.fetchall()
+
+        await cursor.close()
+        for row in rows:
+            coins.add(self.row_to_coin_state(row))
+        return list(coins)
+
+    async def rollback_to_block(self, block_index: int) -> List[CoinRecord]:
         """
         Note that block_index can be negative, in which case everything is rolled back
+        Returns the list of coin records that have been modified
         """
         # Update memory cache
         delete_queue: bytes32 = []
@@ -242,50 +353,78 @@ class CoinStore:
         for coin_name in delete_queue:
             self.coin_record_cache.remove(coin_name)
 
+        coin_changes: Dict[bytes32, CoinRecord] = {}
+        cursor_deleted = await self.coin_record_db.execute(
+            "SELECT * FROM coin_record WHERE confirmed_index>?", (block_index,)
+        )
+        rows = await cursor_deleted.fetchall()
+        for row in rows:
+            coin = Coin(bytes32(bytes.fromhex(row[6])), bytes32(bytes.fromhex(row[5])), uint64.from_bytes(row[7]))
+            record = CoinRecord(coin, uint32(0), row[2], row[3], row[4], uint64(0))
+            coin_changes[record.name] = record
+        await cursor_deleted.close()
+
         # Delete from storage
         c1 = await self.coin_record_db.execute("DELETE FROM coin_record WHERE confirmed_index>?", (block_index,))
         await c1.close()
+
+        cursor_unspent = await self.coin_record_db.execute(
+            "SELECT * FROM coin_record WHERE confirmed_index>?", (block_index,)
+        )
+        rows = await cursor_unspent.fetchall()
+        for row in rows:
+            coin = Coin(bytes32(bytes.fromhex(row[6])), bytes32(bytes.fromhex(row[5])), uint64.from_bytes(row[7]))
+            record = CoinRecord(coin, row[1], uint32(0), False, row[4], row[8])
+            if record.name not in coin_changes:
+                coin_changes[record.name] = record
+        await cursor_unspent.close()
+
         c2 = await self.coin_record_db.execute(
             "UPDATE coin_record SET spent_index = 0, spent = 0 WHERE spent_index>?",
             (block_index,),
         )
         await c2.close()
+        return list(coin_changes.values())
 
     # Store CoinRecord in DB and ram cache
-    async def _add_coin_record(self, record: CoinRecord, allow_replace: bool) -> None:
-        if self.coin_record_cache.get(record.coin.name()) is not None:
-            self.coin_record_cache.remove(record.coin.name())
+    async def _add_coin_records(self, records: List[CoinRecord]) -> None:
 
-        cursor = await self.coin_record_db.execute(
-            f"INSERT {'OR REPLACE ' if allow_replace else ''}INTO coin_record VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?)",
-            (
-                record.coin.name().hex(),
-                record.confirmed_block_index,
-                record.spent_block_index,
-                int(record.spent),
-                int(record.coinbase),
-                str(record.coin.puzzle_hash.hex()),
-                str(record.coin.parent_coin_info.hex()),
-                bytes(record.coin.amount),
-                record.timestamp,
-            ),
+        values = []
+        for record in records:
+            self.coin_record_cache.put(record.coin.name(), record)
+            values.append(
+                (
+                    record.coin.name().hex(),
+                    record.confirmed_block_index,
+                    record.spent_block_index,
+                    int(record.spent),
+                    int(record.coinbase),
+                    str(record.coin.puzzle_hash.hex()),
+                    str(record.coin.parent_coin_info.hex()),
+                    bytes(record.coin.amount),
+                    record.timestamp,
+                )
+            )
+
+        cursor = await self.coin_record_db.executemany(
+            "INSERT INTO coin_record VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?)",
+            values,
         )
         await cursor.close()
 
     # Update coin_record to be spent in DB
-    async def _set_spent(self, coin_name: bytes32, index: uint32) -> uint64:
-        current: Optional[CoinRecord] = await self.get_coin_record(coin_name)
-        if current is None:
-            raise ValueError(f"Cannot spend a coin that does not exist in db: {coin_name}")
+    async def _set_spent(self, coin_names: List[bytes32], index: uint32):
 
-        assert not current.spent  # Redundant sanity check, already checked in block_body_validation
-        spent: CoinRecord = CoinRecord(
-            current.coin,
-            current.confirmed_block_index,
-            index,
-            True,
-            current.coinbase,
-            current.timestamp,
-        )  # type: ignore # noqa
-        await self._add_coin_record(spent, True)
-        return current.coin.amount
+        # if this coin is in the cache, mark it as spent in there
+        updates = []
+        for coin_name in coin_names:
+            r = self.coin_record_cache.get(coin_name)
+            if r is not None:
+                self.coin_record_cache.put(
+                    r.name, CoinRecord(r.coin, r.confirmed_block_index, index, True, r.coinbase, r.timestamp)
+                )
+            updates.append((index, coin_name.hex()))
+
+        await self.coin_record_db.executemany(
+            "UPDATE OR FAIL coin_record SET spent=1,spent_index=? WHERE coin_name=?", updates
+        )
