@@ -8,6 +8,8 @@ from aiohttp import WSCloseCode, WSMessage, WSMsgType
 
 from chia.cmds.init_funcs import chia_full_version_str
 from chia.protocols.protocol_message_types import ProtocolMessageTypes
+from chia.protocols.protocol_state_machine import message_response_ok
+from chia.protocols.protocol_timing import INTERNAL_PROTOCOL_ERROR_BAN_SECONDS
 from chia.protocols.shared_protocol import Capability, Handshake
 from chia.server.outbound_message import Message, NodeType, make_msg
 from chia.server.rate_limits import RateLimiter
@@ -103,6 +105,9 @@ class WSChiaConnection:
         self.outbound_rate_limiter = RateLimiter(incoming=False, percentage_of_limit=outbound_rate_limit_percent)
         self.inbound_rate_limiter = RateLimiter(incoming=True, percentage_of_limit=inbound_rate_limit_percent)
 
+        # Used by crawler/dns introducer
+        self.version = None
+
     async def perform_handshake(self, network_id: str, protocol_version: str, server_port: int, local_type: NodeType):
         if self.is_outbound:
             outbound_handshake = make_msg(
@@ -135,6 +140,8 @@ class WSChiaConnection:
             if inbound_handshake.network_id != network_id:
                 raise ProtocolError(Err.INCOMPATIBLE_NETWORK_ID)
 
+            self.version = inbound_handshake.software_version
+
             self.peer_server_port = inbound_handshake.server_port
             self.connection_type = NodeType(inbound_handshake.node_type)
 
@@ -212,6 +219,12 @@ class WSChiaConnection:
             raise
         self.close_callback(self, ban_time)
 
+    async def ban_peer_bad_protocol(self, log_err_msg: str):
+        """Ban peer for protocol violation"""
+        ban_seconds = INTERNAL_PROTOCOL_ERROR_BAN_SECONDS
+        self.log.error(f"Banning peer for {ban_seconds} seconds: {self.peer_host} {log_err_msg}")
+        await self.close(ban_seconds, WSCloseCode.PROTOCOL_ERROR, Err.INVALID_PROTOCOL_MESSAGE)
+
     def cancel_pending_timeouts(self):
         for _, task in self.pending_timeouts.items():
             task.cancel()
@@ -269,14 +282,22 @@ class WSChiaConnection:
             if attribute is None:
                 raise AttributeError(f"Node type {self.connection_type} does not have method {attr_name}")
 
-            msg = Message(uint8(getattr(ProtocolMessageTypes, attr_name).value), None, args[0])
+            msg: Message = Message(uint8(getattr(ProtocolMessageTypes, attr_name).value), None, args[0])
             request_start_t = time.time()
-            result = await self.create_request(msg, timeout)
+            result = await self.send_request(msg, timeout)
             self.log.debug(
                 f"Time for request {attr_name}: {self.get_peer_logging()} = {time.time() - request_start_t}, "
                 f"None? {result is None}"
             )
             if result is not None:
+                sent_message_type = ProtocolMessageTypes(msg.type)
+                recv_message_type = ProtocolMessageTypes(result.type)
+                if not message_response_ok(sent_message_type, recv_message_type):
+                    # peer protocol violation
+                    error_message = f"WSConnection.invoke sent message {sent_message_type.name} "
+                    f"but received {recv_message_type.name}"
+                    await self.ban_peer_bad_protocol(self.error_message)
+                    raise ProtocolError(Err.INVALID_PROTOCOL_MESSAGE, [error_message])
                 ret_attr = getattr(class_for_type(self.local_type), ProtocolMessageTypes(result.type).name, None)
 
                 req_annotations = ret_attr.__annotations__
@@ -292,7 +313,7 @@ class WSChiaConnection:
 
         return invoke
 
-    async def create_request(self, message_no_id: Message, timeout: int) -> Optional[Message]:
+    async def send_request(self, message_no_id: Message, timeout: int) -> Optional[Message]:
         """Sends a message and waits for a response."""
         if self.closed:
             return None
@@ -461,6 +482,10 @@ class WSChiaConnection:
             await asyncio.sleep(3)
         return None
 
+    # Used by crawler/dns introducer
+    def get_version(self):
+        return self.version
+
     def get_peer_info(self) -> Optional[PeerInfo]:
         result = self.ws._writer.transport.get_extra_info("peername")
         if result is None:
