@@ -1,8 +1,10 @@
 import datetime
+import os
 from pathlib import Path
-from typing import Any, Tuple
+from typing import Any, List, Tuple
 
 import pkg_resources
+from chia.util.ssl import DEFAULT_PERMISSIONS_CERT_FILE, DEFAULT_PERMISSIONS_KEY_FILE
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes, serialization
@@ -22,6 +24,27 @@ def get_mozilla_ca_crt() -> str:
     return str(mozilla_path)
 
 
+def write_ssl_cert_and_key(cert_path: Path, cert_data: bytes, key_path: Path, key_data: bytes):
+    flags = os.O_CREAT | os.O_EXCL | os.O_WRONLY
+
+    for path, data, mode in [
+        (cert_path, cert_data, DEFAULT_PERMISSIONS_CERT_FILE),
+        (key_path, key_data, DEFAULT_PERMISSIONS_KEY_FILE),
+    ]:
+        if path.exists():
+            path.unlink()
+
+        with open(os.open(str(path), flags, mode), "wb") as f:
+            f.write(data)  # lgtm [py/clear-text-storage-sensitive-data]
+
+
+def ensure_ssl_dirs(dirs: List[Path]):
+    """Create SSL dirs with a default 755 mode if necessary"""
+    for dir in dirs:
+        if not dir.exists():
+            dir.mkdir(mode=0o755)
+
+
 def generate_ca_signed_cert(ca_crt: bytes, ca_key: bytes, cert_out: Path, key_out: Path):
     one_day = datetime.timedelta(1, 0, 0)
     root_cert = x509.load_pem_x509_certificate(ca_crt, default_backend())
@@ -58,8 +81,7 @@ def generate_ca_signed_cert(ca_crt: bytes, ca_key: bytes, cert_out: Path, key_ou
         encryption_algorithm=serialization.NoEncryption(),
     )
 
-    cert_out.write_bytes(cert_pem)
-    key_out.write_bytes(key_pem)
+    write_ssl_cert_and_key(cert_out, cert_pem, key_out, key_pem)
 
 
 def make_ca_cert(cert_path: Path, key_path: Path):
@@ -83,19 +105,14 @@ def make_ca_cert(cert_path: Path, key_path: Path):
         .sign(root_key, hashes.SHA256(), default_backend())
     )
 
-    cert_path.write_bytes(
-        root_cert.public_bytes(
-            encoding=serialization.Encoding.PEM,
-        )
+    cert_pem = root_cert.public_bytes(encoding=serialization.Encoding.PEM)
+    key_pem = root_key.private_bytes(
+        encoding=serialization.Encoding.PEM,
+        format=serialization.PrivateFormat.TraditionalOpenSSL,
+        encryption_algorithm=serialization.NoEncryption(),
     )
 
-    key_path.write_bytes(
-        root_key.private_bytes(
-            encoding=serialization.Encoding.PEM,
-            format=serialization.PrivateFormat.TraditionalOpenSSL,
-            encryption_algorithm=serialization.NoEncryption(),
-        )
-    )
+    write_ssl_cert_and_key(cert_path, cert_pem, key_path, key_pem)
 
 
 def main():
