@@ -1,5 +1,7 @@
 from dataclasses import dataclass
+from typing import Optional
 
+from chia.protocols.wallet_protocol import CoinState
 from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.ints import uint32, uint64
@@ -24,3 +26,13 @@ class CoinRecord(Streamable):
     @property
     def name(self) -> bytes32:
         return self.coin.name()
+
+    @property
+    def coin_state(self) -> CoinState:
+        spent_h = None
+        if self.spent:
+            spent_h = self.spent_block_index
+        confirmed_height: Optional[uint32] = self.confirmed_block_index
+        if self.confirmed_block_index == 0 and self.timestamp == 0:
+            confirmed_height = None
+        return CoinState(self.coin, spent_h, confirmed_height)
