@@ -124,32 +124,6 @@ def puzzle_announcements_for_conditions_dict(
     return output_announcements
 
 
-def coin_announcements_names_for_npc(npc_list) -> Set[bytes32]:
-    output_announcements: Set[bytes32] = set()
-    for npc in npc_list:
-        for condition, cvp_list in npc.conditions:
-            if condition == ConditionOpcode.CREATE_COIN_ANNOUNCEMENT:
-                for cvp in cvp_list:
-                    message = cvp.vars[0]
-                    assert len(message) <= 1024
-                    announcement = Announcement(npc.coin_name, message)
-                    output_announcements.add(announcement.name())
-    return output_announcements
-
-
-def puzzle_announcements_names_for_npc(npc_list) -> Set[bytes32]:
-    output_announcements: Set[bytes32] = set()
-    for npc in npc_list:
-        for condition, cvp_list in npc.conditions:
-            if condition == ConditionOpcode.CREATE_PUZZLE_ANNOUNCEMENT:
-                for cvp in cvp_list:
-                    message = cvp.vars[0]
-                    assert len(message) <= 1024
-                    announcement = Announcement(npc.puzzle_hash, message)
-                    output_announcements.add(announcement.name())
-    return output_announcements
-
-
 def coin_announcement_names_for_conditions_dict(
     conditions_dict: Dict[ConditionOpcode, List[ConditionWithArgs]],
     input_coin: Coin,
