@@ -1,8 +1,10 @@
+import asyncio
 import copy
 import logging
 import os
 import random
 import shutil
+import ssl
 import sys
 import tempfile
 import time
@@ -15,6 +17,7 @@ from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 from chiabip158 import PyBIP158
 
 from chia.cmds.init_funcs import create_all_ssl, create_default_chia_config
+from chia.daemon.keychain_proxy import connect_to_keychain_and_validate, wrap_local_keychain
 from chia.full_node.bundle_tools import (
     best_solution_generator_from_template,
     detect_potential_template_generator,
@@ -23,18 +26,18 @@ from chia.full_node.bundle_tools import (
 from chia.util.errors import Err
 from chia.full_node.generator import setup_generator_args
 from chia.full_node.mempool_check_conditions import GENERATOR_MOD
-from chia.plotting.create_plots import create_plots
+from chia.plotting.create_plots import create_plots, PlotKeys
 from chia.consensus.block_creation import unfinished_block_to_full_block
 from chia.consensus.block_record import BlockRecord
 from chia.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
 from chia.consensus.blockchain_interface import BlockchainInterface
 from chia.consensus.coinbase import create_puzzlehash_for_pk, create_farmer_coin, create_pool_coin
+from chia.consensus.condition_costs import ConditionCost
 from chia.consensus.constants import ConsensusConstants
 from chia.consensus.default_constants import DEFAULT_CONSTANTS
 from chia.consensus.deficit import calculate_deficit
 from chia.consensus.full_block_to_block_record import block_to_block_record
 from chia.consensus.make_sub_epoch_summary import next_sub_epoch_summary
-from chia.consensus.cost_calculator import NPCResult, calculate_cost_of_program
 from chia.consensus.pot_iterations import (
     calculate_ip_iters,
     calculate_iterations_quality,
@@ -44,11 +47,14 @@ from chia.consensus.pot_iterations import (
 )
 from chia.consensus.vdf_info_computation import get_signage_point_vdf_info
 from chia.full_node.signage_point import SignagePoint
-from chia.plotting.plot_tools import PlotInfo, load_plots, parse_plot_info
+from chia.plotting.util import PlotInfo, PlotsRefreshParameter, PlotRefreshResult, parse_plot_info
+from chia.plotting.manager import PlotManager
+from chia.server.server import ssl_context_for_server
 from chia.types.blockchain_format.classgroup import ClassgroupElement
 from chia.types.blockchain_format.coin import Coin, hash_coin_list
 from chia.types.blockchain_format.foliage import Foliage, FoliageBlockData, FoliageTransactionBlock, TransactionsInfo
 from chia.types.blockchain_format.pool_target import PoolTarget
+from chia.types.blockchain_format.program import INFINITE_COST
 from chia.types.blockchain_format.proof_of_space import ProofOfSpace
 from chia.types.blockchain_format.reward_chain_block import RewardChainBlockUnfinished
 from chia.types.blockchain_format.sized_bytes import bytes32
@@ -60,16 +66,14 @@ from chia.types.blockchain_format.slots import (
 )
 from chia.types.blockchain_format.sub_epoch_summary import SubEpochSummary
 from chia.types.blockchain_format.vdf import VDFInfo, VDFProof
-from chia.types.condition_with_args import ConditionWithArgs
 from chia.types.end_of_slot_bundle import EndOfSubSlotBundle
 from chia.types.full_block import FullBlock
 from chia.types.generator_types import BlockGenerator, CompressorArg
 from chia.types.spend_bundle import SpendBundle
 from chia.types.unfinished_block import UnfinishedBlock
-from chia.types.name_puzzle_condition import NPC
 from chia.util.bech32m import encode_puzzle_hash
 from chia.util.block_cache import BlockCache
-from chia.util.condition_tools import ConditionOpcode, conditions_by_opcode
+from chia.util.condition_tools import ConditionOpcode
 from chia.util.config import load_config, save_config
 from chia.util.hash import std_hash
 from chia.util.ints import uint8, uint16, uint32, uint64, uint128
@@ -108,7 +112,6 @@ test_constants = DEFAULT_CONSTANTS.replace(
         * 10,  # Allows creating blockchains with timestamps up to 10 days in the future, for testing
         "COST_PER_BYTE": 1337,
         "MEMPOOL_BLOCK_BUFFER": 6,
-        "INITIAL_FREEZE_END_TIMESTAMP": 1000,
         "NETWORK_TYPE": 1,
     }
 )
@@ -123,7 +126,11 @@ class BlockTools:
     """
 
     def __init__(
-        self, constants: ConsensusConstants = test_constants, root_path: Optional[Path] = None, const_dict=None
+        self,
+        constants: ConsensusConstants = test_constants,
+        root_path: Optional[Path] = None,
+        const_dict=None,
+        keychain: Optional[Keychain] = None,
     ):
         self._tempdir = None
         if root_path is None:
@@ -131,33 +138,11 @@ class BlockTools:
             root_path = Path(self._tempdir.name)
 
         self.root_path = root_path
-        create_default_chia_config(root_path)
-        self.keychain = Keychain("testing-1.8.0", True)
-        self.keychain.delete_all_keys()
-        self.farmer_master_sk_entropy = std_hash(b"block_tools farmer key")
-        self.pool_master_sk_entropy = std_hash(b"block_tools pool key")
-        self.farmer_master_sk = self.keychain.add_private_key(bytes_to_mnemonic(self.farmer_master_sk_entropy), "")
-        self.pool_master_sk = self.keychain.add_private_key(bytes_to_mnemonic(self.pool_master_sk_entropy), "")
-        self.farmer_pk = master_sk_to_farmer_sk(self.farmer_master_sk).get_g1()
-        self.pool_pk = master_sk_to_pool_sk(self.pool_master_sk).get_g1()
-        self.farmer_ph: bytes32 = create_puzzlehash_for_pk(
-            master_sk_to_wallet_sk(self.farmer_master_sk, uint32(0)).get_g1()
-        )
-        self.pool_ph: bytes32 = create_puzzlehash_for_pk(
-            master_sk_to_wallet_sk(self.pool_master_sk, uint32(0)).get_g1()
-        )
-        self.init_plots(root_path)
+        self.local_keychain = keychain
 
+        create_default_chia_config(root_path)
         create_all_ssl(root_path)
 
-        self.all_sks: List[PrivateKey] = [sk for sk, _ in self.keychain.get_all_private_keys()]
-        self.pool_pubkeys: List[G1Element] = [master_sk_to_pool_sk(sk).get_g1() for sk in self.all_sks]
-
-        self.farmer_pubkeys: List[G1Element] = [master_sk_to_farmer_sk(sk).get_g1() for sk in self.all_sks]
-        if len(self.pool_pubkeys) == 0 or len(self.farmer_pubkeys) == 0:
-            raise RuntimeError("Keys not generated. Run `chia generate keys`")
-
-        self.load_plots()
         self.local_sk_cache: Dict[bytes32, Tuple[PrivateKey, Any]] = {}
         self._config = load_config(self.root_path, "config.yaml")
         self._config["logging"]["log_stdout"] = True
@@ -171,6 +156,38 @@ class BlockTools:
             updated_constants = updated_constants.replace(**const_dict)
         self.constants = updated_constants
 
+    async def setup_keys(self):
+        if self.local_keychain:
+            self.keychain_proxy = wrap_local_keychain(self.local_keychain, log=log)
+        else:
+            self.keychain_proxy = await connect_to_keychain_and_validate(
+                self.root_path, log, user="testing-1.8.0", service="chia-testing-1.8.0"
+            )
+
+        await self.keychain_proxy.delete_all_keys()
+        self.farmer_master_sk_entropy = std_hash(b"block_tools farmer key")
+        self.pool_master_sk_entropy = std_hash(b"block_tools pool key")
+        self.farmer_master_sk = await self.keychain_proxy.add_private_key(
+            bytes_to_mnemonic(self.farmer_master_sk_entropy), ""
+        )
+        self.pool_master_sk = await self.keychain_proxy.add_private_key(
+            bytes_to_mnemonic(self.pool_master_sk_entropy), ""
+        )
+        self.farmer_pk = master_sk_to_farmer_sk(self.farmer_master_sk).get_g1()
+        self.pool_pk = master_sk_to_pool_sk(self.pool_master_sk).get_g1()
+        self.farmer_ph: bytes32 = create_puzzlehash_for_pk(
+            master_sk_to_wallet_sk(self.farmer_master_sk, uint32(0)).get_g1()
+        )
+        self.pool_ph: bytes32 = create_puzzlehash_for_pk(
+            master_sk_to_wallet_sk(self.pool_master_sk, uint32(0)).get_g1()
+        )
+        self.all_sks: List[PrivateKey] = [sk for sk, _ in await self.keychain_proxy.get_all_private_keys()]
+        self.pool_pubkeys: List[G1Element] = [master_sk_to_pool_sk(sk).get_g1() for sk in self.all_sks]
+
+        self.farmer_pubkeys: List[G1Element] = [master_sk_to_farmer_sk(sk).get_g1() for sk in self.all_sks]
+        if len(self.pool_pubkeys) == 0 or len(self.farmer_pubkeys) == 0:
+            raise RuntimeError("Keys not generated. Run `chia generate keys`")
+
     def change_config(self, new_config: Dict):
         self._config = new_config
         overrides = self._config["network_overrides"]["constants"][self._config["selected_network"]]
@@ -178,11 +195,7 @@ class BlockTools:
         self.constants = updated_constants
         save_config(self.root_path, "config.yaml", self._config)
 
-    def load_plots(self):
-        _, loaded_plots, _, _ = load_plots({}, {}, self.farmer_pubkeys, self.pool_pubkeys, None, False, self.root_path)
-        self.plots: Dict[Path, PlotInfo] = loaded_plots
-
-    def init_plots(self, root_path: Path):
+    async def setup_plots(self):
         plot_dir = get_plot_dir()
         mkdir(plot_dir)
         temp_dir = get_plot_tmp_dir()
@@ -195,9 +208,6 @@ class BlockTools:
         # Uses many plots for testing, in order to guarantee proofs of space at every height
         args.num = num_pool_public_key_plots  # Some plots created to a pool public key, and some to a pool puzzle hash
         args.buffer = 100
-        args.farmer_public_key = bytes(self.farmer_pk).hex()
-        args.pool_public_key = bytes(self.pool_pk).hex()
-        args.pool_contract_address = None
         args.tmp_dir = temp_dir
         args.tmp2_dir = plot_dir
         args.final_dir = plot_dir
@@ -214,20 +224,23 @@ class BlockTools:
             for i in range(num_pool_public_key_plots + num_pool_address_plots)
         ]
         try:
+            plot_keys_1 = PlotKeys(self.farmer_pk, self.pool_pk, None)
+
             # No datetime in the filename, to get deterministic filenames and not re-plot
-            create_plots(
+            await create_plots(
                 args,
-                root_path,
+                plot_keys_1,
+                self.root_path,
                 use_datetime=False,
                 test_private_keys=test_private_keys[:num_pool_public_key_plots],
             )
             # Create more plots, but to a pool address instead of public key
-            args.pool_public_key = None
-            args.pool_contract_address = encode_puzzle_hash(self.pool_ph, "xch")
+            plot_keys_2 = PlotKeys(self.farmer_pk, None, encode_puzzle_hash(self.pool_ph, "xch"))
             args.num = num_pool_address_plots
-            create_plots(
+            await create_plots(
                 args,
-                root_path,
+                plot_keys_2,
+                self.root_path,
                 use_datetime=False,
                 test_private_keys=test_private_keys[num_pool_public_key_plots:],
             )
@@ -235,10 +248,49 @@ class BlockTools:
             shutil.rmtree(plot_dir, ignore_errors=True)
             sys.exit(1)
 
+        refresh_done = False
+        refresh_parameter: PlotsRefreshParameter = PlotsRefreshParameter(batch_size=2)
+
+        def test_callback(update_result: PlotRefreshResult):
+            if update_result.remaining_files == 0:
+                nonlocal refresh_done
+                refresh_done = True
+            else:
+                assert 0 < update_result.loaded_plots <= refresh_parameter.batch_size
+                assert update_result.loaded_plots == update_result.processed_files
+                assert update_result.remaining_files > 0
+                assert update_result.loaded_size > 0
+                assert 0 < update_result.duration < 5
+
+        plot_manager: PlotManager = PlotManager(
+            self.root_path, refresh_parameter=refresh_parameter, refresh_callback=test_callback
+        )
+        plot_manager.set_public_keys(self.farmer_pubkeys, self.pool_pubkeys)
+        plot_manager.start_refreshing()
+
+        retry = 5
+        while not refresh_done and retry > 0:
+            time.sleep(1)
+            retry -= 1
+        assert refresh_done
+        plot_manager.stop_refreshing()
+        self.plots: Dict[Path, PlotInfo] = plot_manager.plots
+        # create_plots() updates plot_directories. Ensure we refresh our config to reflect the updated value
+        self._config["harvester"]["plot_directories"] = load_config(self.root_path, "config.yaml", "harvester")[
+            "plot_directories"
+        ]
+
     @property
     def config(self) -> Dict:
         return copy.deepcopy(self._config)
 
+    def get_daemon_ssl_context(self) -> Optional[ssl.SSLContext]:
+        crt_path = self.root_path / self.config["daemon_ssl"]["private_crt"]
+        key_path = self.root_path / self.config["daemon_ssl"]["private_key"]
+        ca_cert_path = self.root_path / self.config["private_ssl_ca"]["crt"]
+        ca_key_path = self.root_path / self.config["private_ssl_ca"]["key"]
+        return ssl_context_for_server(ca_cert_path, ca_key_path, crt_path, key_path)
+
     def get_plot_signature(self, m: bytes32, plot_pk: G1Element) -> G2Element:
         """
         Returns the plot signature of the header data.
@@ -1417,43 +1469,26 @@ def get_full_block_and_block_record(
     return full_block, block_record
 
 
-def get_name_puzzle_conditions_test(generator: BlockGenerator, max_cost: int) -> NPCResult:
-    """
-    This is similar to get_name_puzzle_conditions(), but it doesn't validate
-    the conditions. We rely on this in tests to create invalid blocks.
-    safe_mode is implicitly True in this call
-    """
+def compute_cost_test(generator: BlockGenerator, cost_per_byte: int) -> Tuple[Optional[uint16], uint64]:
     try:
         block_program, block_program_args = setup_generator_args(generator)
-        clvm_cost, result = GENERATOR_MOD.run_safe_with_cost(max_cost, block_program, block_program_args)
-
-        npc_list: List[NPC] = []
+        clvm_cost, result = GENERATOR_MOD.run_safe_with_cost(INFINITE_COST, block_program, block_program_args)
+        size_cost = len(bytes(generator.program)) * cost_per_byte
+        condition_cost = 0
 
         for res in result.first().as_iter():
-            conditions_list: List[ConditionWithArgs] = []
-
-            spent_coin_parent_id: bytes32 = res.first().as_atom()
-            res = res.rest()
-            spent_coin_puzzle_hash: bytes32 = res.first().as_atom()
-            res = res.rest()
-            spent_coin_amount: uint64 = uint64(res.first().as_int())
-            res = res.rest()
-            spent_coin: Coin = Coin(spent_coin_parent_id, spent_coin_puzzle_hash, spent_coin_amount)
-
+            res = res.rest()  # skip parent coind id
+            res = res.rest()  # skip puzzle hash
+            res = res.rest()  # skip amount
             for cond in res.first().as_iter():
                 condition = cond.first().as_atom()
-                cvl = ConditionWithArgs(ConditionOpcode(condition), cond.rest().as_atom_list())
-                conditions_list.append(cvl)
-
-            conditions_dict = conditions_by_opcode(conditions_list)
-            if conditions_dict is None:
-                conditions_dict = {}
-            npc_list.append(
-                NPC(spent_coin.name(), spent_coin.puzzle_hash, [(a, b) for a, b in conditions_dict.items()])
-            )
-        return NPCResult(None, npc_list, uint64(clvm_cost))
+                if condition in [ConditionOpcode.AGG_SIG_UNSAFE, ConditionOpcode.AGG_SIG_ME]:
+                    condition_cost += ConditionCost.AGG_SIG.value
+                elif condition == ConditionOpcode.CREATE_COIN:
+                    condition_cost += ConditionCost.CREATE_COIN.value
+        return None, uint64(clvm_cost + size_cost + condition_cost)
     except Exception:
-        return NPCResult(uint16(Err.GENERATOR_RUNTIME_ERROR.value), [], uint64(0))
+        return uint16(Err.GENERATOR_RUNTIME_ERROR.value), uint64(0)
 
 
 def create_test_foliage(
@@ -1547,8 +1582,8 @@ def create_test_foliage(
         # Calculate the cost of transactions
         if block_generator is not None:
             generator_block_heights_list = block_generator.block_height_list()
-            result: NPCResult = get_name_puzzle_conditions_test(block_generator, constants.MAX_BLOCK_COST_CLVM)
-            cost = calculate_cost_of_program(block_generator.program, result, constants.COST_PER_BYTE)
+            err, cost = compute_cost_test(block_generator, constants.COST_PER_BYTE)
+            assert err is None
 
             removal_amount = 0
             addition_amount = 0
@@ -1836,3 +1871,30 @@ def create_test_unfinished_block(
         block_generator.program if block_generator else None,
         block_generator.block_height_list() if block_generator else [],
     )
+
+
+async def create_block_tools_async(
+    constants: ConsensusConstants = test_constants,
+    root_path: Optional[Path] = None,
+    const_dict=None,
+    keychain: Optional[Keychain] = None,
+) -> BlockTools:
+    bt = BlockTools(constants, root_path, const_dict, keychain)
+    await bt.setup_keys()
+    await bt.setup_plots()
+
+    return bt
+
+
+def create_block_tools(
+    constants: ConsensusConstants = test_constants,
+    root_path: Optional[Path] = None,
+    const_dict=None,
+    keychain: Optional[Keychain] = None,
+) -> BlockTools:
+    bt = BlockTools(constants, root_path, const_dict, keychain)
+
+    asyncio.get_event_loop().run_until_complete(bt.setup_keys())
+    asyncio.get_event_loop().run_until_complete(bt.setup_plots())
+
+    return bt
